##############################################################################
#
# Combined landot AAS and MenDAKE's Seach and Destroy
# 
# For BF2, Special Forces, Euroforce and Armored Fury
#
# Description: This mod contains two game modes: Advance and Secure and 
# Search and Destroy. AAS requires flags to be captured in succession, 
# concentrating the action on one or two flags at a time. Search and 
# Destroy is an objective based game where one team must defend their 
# strategic objectives (i.e. Radar, UAV, Artillery) and the other must
# destroy them.
#
# These modes can be played on their own, or in combination.
# When played in combination each round alternates between AAS and SD. The 
# team that wins the AAS game will autmoatically be the SD attackers in the 
# next round.
#
#############################################################################
#
# Version 1.0 (beta)
#
# The following copyright only applies to pieces of code added or changed
# in this file compared to the original gpm_cq.py included with BF2 1.02.
# Any other code is copyright of the copyright owner of that particular code.
#
# (AAS) Copyright (c) 2005, Jarkko Santala <santala@landot.org>
# (SD) Copyright (c) 2005, MenDAKE <mendake_ddude@yahoo.com>
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
# * Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
# * Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
# TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
# PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
# CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
# EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
# OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
# WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
# OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
# ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
# Contributors:
#
# MenDAKE
#
# Please check SETTINGS below for both game modes.
#
#############################################################################

TAKEOVERTYPE_CAPTURE = 1
TAKEOVERTYPE_NEUTRALIZE = 2

SCORE_CAPTURE = 2
SCORE_NEUTRALIZE = 2
SCORE_CAPTUREASSIST = 1
SCORE_NEUTRALIZEASSIST = 1
SCORE_DEFEND = 1

Top = 0
Middle = 1
Bottom = 2

import host
import bf2
import math
from game.scoringCommon import addScore, RPL
from bf2 import g_debug

g_controlPoints = [] # cache, as this map won't change

# AAS globals.
global aasTip
global aasLast
global aasNextCp
global aasRadius
global aasVehicleCap
global aasCpBalance
global aasDebug
global aasL10n
global aasL10nCp
global aasL10nText
global aasNoRatio

# SD globals.
global sdLast
global sdNextCp
global sdVehicleCap
global sdDebug
global sdL10n
global sdL10nText
global sdDefendingTeam
global sdNoRatio
global sdAttackingTeam
global initialGameMode
global gameMode
global aasWinner
global sdAttackerTicketFactor

####################################################################################
#
# General game settings affecting both game modes.
#
# Sets the initial game mode. "aas" = "Advance and Destroy" mode only, "sd" = "Search
# and Destroy" mode only. "both" = first "Advance and Secure" then "Search and Destroy".
# If set to both, then there must be an even number of rounds (2 minimum).
#
initialGameMode = "both"
#
####################################################################################

###############################################################################
#
# landot AAS mod - SETTINGS
#
#
# The CP area sizes will be calculated using the following formula:
#
# Original CP Radius * ( Max Number Of Players / aasRadius )
#
# If you set the aasRadius to 0 the CP area sizes will stay normal.
# With default setting 10:  10 players = the same, 30 players = 3 times bigger
#
aasRadius = 0
#
# Choose localization by setting the variable aasL10n
#
# 0 = English
# 1 = French
# 2 = German
#
aasL10n = 0
#
# Enable/disable Commander
#
# 1 = Enable
# 0 = Disable
#
aasCommander = 1
#
# Using the following setting you can choose whether CPs per team will
# be balanced at the start of each map. Enabling this setting means that
# the battle will always start in the middle of the map.
#
# True  = Balance number of CPs per team at map start
# False = Don't balance, use default CP ownerships
#
aasCpBalance = True
#
# If set to true the ticket ratio server option acts as an exact number of 
# tickets instead of a percentage. This keeps the ticket count from increasing
# with larger maps.
#
aasNoRatio = True
#
# No user serviceable parts below this point!
#
###############################################################################

###############################################################################
#
# Search and Destroy Server Settings
#
# Choose localization by setting the variable sdL10n
#
# 0 = English
# 1 = French
# 2 = German
#
sdL10n = 0
#
# This option gives the attackers less tickets. This is especially useful when
# there is no time limit and ticket count is what decides when the defenders win.
# (Usage: This is percentage of the total tickets. For example, 50 will reduce total 
# attaker's tickets by 50 percent. 0 will do nothing.
#
sdAttackerTicketFactor = 0
#
# Enable/disable Commander
#
# 1 = Enable
# 0 = Disable
#
sdCommander = 1
#
# If set to true the ticket ratio server option acts as an exact number 
# of tickets instead of a percentage. This keeps the ticket count from increasing
# with larger maps.
#
sdNoRatio = True
#
# Sets the default attacking team, in case only SD mode is played. Normally the SD
# attacking team is the one who won the last round of aas.
#
sdAttackingTeam = 1
#
####################################################################################

aasDebug = 0
aasLast = [ 0, 0, 0 ]
aasNextCp = [ 0, 0, 0 ]

aasTip = " * "

aasL10nCp = { "abandonedhuts" : [ "Abandoned Huts" , "Baraquements dsaffects", "Verlassene Htten" ] ,
	      "abandonedvillage" : [ "Abandoned Village" , "Village abandonn", "Verlassenes Dorf" ] ,
	      "airfield" : [ "Airfield" , "Arodrome", "Flugplatz" ] ,
	      "airfieldbase" : [ "Airfield Base" , "Base de l arodrome", "Basis am Flugplatz" ] ,
	      "airfieldcamp" : [ "Airfield Camp" , "Camp de l arodrome", "Lager am Flugplatz" ] ,
	      "alley" : [ "Alley" , "Rue", "Gasse" ] ,
	      "alleyway" : [ "Alleyway" , "Ruelle", "Durchgang" ] ,
	      "ammodepot" : [ "Ammo Depot" , "Dpt de munitions", "Munitionslager" ] ,
	      "backyard" : [ "Backyard" , "Arrire-cour", "Hinterhof" ] ,
	      "beach" : [ "Beach" , "Plage", "Strand" ] ,
	      "beachhouse" : [ "Beach House" , "Maison ctire", "Strandhaus" ] ,
	      "bridge" : [ "Bridge" , "Pont", "Brcke" ] ,
	      "bridgebase" : [ "Bridge Base" , "Base du pont", "Brckenbasis" ] ,
	      "bridgecamp" : [ "Bridge Camp" , "Camp du pont", "Brckenlager" ] ,
	      "canyonguardpost" : [ "Canyon Guard Post" , "Poste de garde du dfil", "Wachposten an der Schlucht" ] ,
	      "cementfactory" : [ "Cement Factory" , "Cimenterie", "Zementfabrik" ] ,
	      "centralfront" : [ "Central Front" , "Front central", "Zentrale Front" ] ,
	      "centralisland" : [ "Central Island" , "Ile centrale", "Zentrale Insel" ] ,
	      "chineseairfield" : [ "Chinese Airfield" , "Arodrome chinois", "Flugplatz (China)" ] ,
	      "chinesebase" : [ "Chinese Base" , "Base chinoise", "Basis (China)" ] ,
	      "chinesehq" : [ "Chinese HQ" , "PC chinois", "HQ (China)" ] ,
	      "city" : [ "City" , "Ville", "Stadt" ] ,
	      "city_entrance" : [ "City Entrance" , "Entre de la ville", "Stadt-Eingang" ] ,
	      "citypark" : [ "City Park" , "Parc", "Stadtpark" ] ,
	      "clubhouse" : [ "Club House" , "Pavillon", "Klubhaus" ] ,
	      "communicationcentral" : [ "Communication Central" , "Central de communication", "Kommunikationszentrale" ] ,
	      "constructionsite" : [ "Construction Site" , "Chantier", "Baustelle" ] ,
	      "containerstorage" : [ "Container Storage" , "Zone de stockage", "Lagerhaus" ] ,
	      "controlcenter" : [ "Control Center" , "Centre de contrle", "Kontrollzentrum" ] ,
	      "convoy" : [ "Convoy" , "Convoi", "Konvoi" ] ,
	      "docks" : [ "Docks" , "Quais", "Docks" ] ,
	      "eastoilfield" : [ "East Oilfield" , "Champ ptrolifre Est", "stliches lfeld" ] ,
	      "emineentrance" : [ "East Mine Entrance" , "Entre Est de la mine", "Eingang zur stlichen Mine" ] ,
	      "factory" : [ "Factory" , "Usine", "Fabrik" ] ,
	      "farm" : [ "Farm" , "Ferme", "Bauernhof" ] ,
	      "fishfactory" : [ "Fish Factory" , "Conserverie de poisson", "Fischfabrik" ] ,
	      "fortification" : [ "Fortification" , "Fortification", "Befestigungsanlage" ] ,
	      "gasstation" : [ "Gas Station" , "Station-service", "Tankstelle" ] ,
	      "gatehouse" : [ "Gatehouse" , "Maison de gardien", "Pfrtnerhaus" ] ,
	      "harbor" : [ "Harbor" , "Port", "Hafen" ] ,
	      "hill" : [ "Hill" , "Colline", "Hgel" ] ,
	      "hillvillage" : [ "Hill Village" , "Village sur la colline", "Bergdorf" ] ,
	      "hotel" : [ "Hotel" , "Htel", "Hotel" ] ,
	      "industrialarea" : [ "Industrial Area" , "Zone industrielle", "Industriegebiet" ] ,
	      "intake" : [ "Intake" , "Chemine", "Einlaufturm" ] ,
	      "island" : [ "Island" , "Ile", "Insel" ] ,
	      "islandfarmhouse" : [ "Island Farmhouse" , "Ferme insulaire", "Insel-Bauernhaus" ] ,
	      "lowerdam" : [ "Lower Dam" , "Pied du barrage", "Unterer Damm" ] ,
	      "mainentrance" : [ "Main Entrance" , "Entre principale", "Haupteingang" ] ,
	      "maintenancebuilding" : [ "Maintenance Buildings" , "Locaux d entretien", "Wartungsgebude" ] ,
	      "market" : [ "Market Place" , "March", "Marktplatz" ] ,
	      "marketplace" : [ "Market Place" , "Place du march", "Marktplatz" ] ,
	      "materialstation" : [ "Material Station" , "Dpt de matriel", "Materialstation" ] ,
	      "mecairfield" : [ "MEC Airfield" , "Arodrome CMO", "Flugplatz (MEC)" ] ,
	      "mecbase" : [ "MEC Base" , "Base CMO", "Basis (MEC)" ] ,
	      "mosque" : [ "Mosque", "Mosque", "Mosque" ] ,
	      "mountainlookout" : [ "Mountain Lookout" , "Poste de guet", "Beobachtungsposten auf dem Berg" ] ,
	      "naturalgasstation" : [ "Natural Gas Station" , "Exploitation de gaz naturel", "Erdgas-Werk" ] ,
	      "ngasstation" : [ "North Gas Station" , "Station-service Nord", "Nrdliche Tankstelle" ] ,
	      "northbeach" : [ "North Beach" , "Plage Nord", "Nrdlicher Strand" ] ,
	      "northdocks" : [ "North Docks" , "Quais Nord", "Nrdliche Docks" ] ,
	      "northfront" : [ "North Front" , "Front Nord", "Nrdliche Front" ] ,
	      "northisland" : [ "North Island" , "Ile septentrionale", "Nrdliche Insel" ] ,
	      "northperimeter" : [ "North Perimeter" , "Nord du primtre", "Randstellung Nord" ] ,
	      "officebuildings" : [ "Office Buildings" , "Bureaux", "Brogebude" ] ,
	      "oilcisterns" : [ "Oil Cisterns" , "Citernes d essence", "l-Zisternen" ] ,
	      "oilfieldcheckpoint" : [ "Oilfield Checkpoint" , "Point de contrle champ ptrol.", "Checkpoint lfeld" ] ,
	      "oldcity" : [ "Old City" , "Vieille ville", "Alte Stadt" ] ,
	      "olivefort" : [ "Olive Hill Fortification" , "Fortif. colline des oliviers", "Olivenberg-Befestigung" ] ,
	      "outpost" : [ "Outpost" , "Avant-poste", "Vorposten" ] ,
	      "outskirts" : [ "Outskirts" , "Faubourgs", "Auenbezirke" ] ,
	      "powerplant" : [ "Power Plant" , "Centrale lectrique", "Elektrizittswerk" ] ,
	      "powerstation" : [ "Power Station" , "Centrale lectrique", "Kraftwerk" ] ,
	      "reactors" : [ "Reactor Towers" , "Chemines", "Reaktoren" ] ,
	      "refinery" : [ "Refinery" , "Raffinerie", "Raffinerie" ] ,
	      "refineryentrance" : [ "Refinery Entrance" , "Entre de la raffinerie", "Eingang zur Raffinerie" ] ,
	      "refuellingstation" : [ "Refueling Station" , "Poste de ravitaillement", "Tankstation" ] ,
	      "riverfort" : [ "River Fortification" , "Fortification fluviale", "Fluss-Befestigung" ] ,
	      "rivervillage" : [ "River Village" , "Village en bord de rivire", "Dorf am Fluss" ] ,
	      "roadblock" : [ "Road Block" , "Barrage routier", "Straensperre" ] ,
	      "roadconstruct" : [ "Road Construction" , "Route en travaux", "Straenbaustelle" ] ,
	      "rockfort" : [ "Rock Fortification" , "Fortification du rocher", "Fels-Befestigung" ] ,
	      "securityhq" : [ "Security Headquarters" , "QG de la scurit", "Sicherheits-Hauptquartier" ] ,
	      "sgasstation" : [ "South Gas Station" , "Station-service Sud", "Sdliche Tankstelle" ] ,
	      "smallairstrip" : [ "Small Airstrip" , "Petite piste d aviation", "Kleines Flugfeld" ] ,
	      "southbeach" : [ "South Beach" , "Plage Sud", "Sdlicher Strand" ] ,
	      "southblock" : [ "South Block" , "Pt de maisons Sud", "Sdlicher Block" ] ,
	      "southdocks" : [ "South Docks" , "Quais Sud", "Sdliche Docks" ] ,
	      "southfront" : [ "South Front" , "Front Sud", "Sdliche Front" ] ,
	      "southisland" : [ "South Island" , "Ile mridionale", "Sdliche Insel" ] ,
	      "southperimeter" : [ "South Perimeter" , "Sud du primtre", "Randstellung Sd" ] ,
	      "square" : [ "Square" , "Place", "Platz" ] ,
	      "suburb" : [ "Suburb" , "Banlieue", "Vorort" ] ,
	      "supervisorbase" : [ "Supervisor Base" , "Base de surveillance", "Basis (Aufseher)" ] ,
	      "surveillancepost" : [ "Surveillance Post" , "Poste de surveillance", "Kontrollposten" ] ,
	      "temple" : [ "Temple" , "Temple", "Tempel" ] ,
	      "trainaccident" : [ "Train Accident" , "Accident ferroviaire", "Zugunglck" ] ,
	      "tvstation" : [ "TV Station" , "Station TV", "Fernsehsender" ] ,
	      "uppercamp" : [ "Upper Camp" , "Camp suprieur", "Oberes Lager" ] ,
	      "upperdam" : [ "Upper Dam" , "Sommet du barrage", "Oberer Damm" ] ,
	      "usairfield" : [ "US Airfield" , "Arodrome amricain", "Flugplatz (US)" ] ,
	      "uscarrier" : [ "US Carrier" , "Porte-avions amricain", "Flugzeugtrger (US)" ] ,
	      "ustown" : [ "US Town" , "Ville amricaine", "Stadt (US)" ] ,
	      "village" : [ "Village" , "Village", "Verlassenes Dorf" ] ,
	      "villagefort" : [ "Village Fortification" , "Fortification du village", "Dorf-Befestigung" ] ,
	      "vistapoint" : [ "Vista Point" , "Point de vue", "Aussichtspunkt" ] ,
	      "warehouse" : [ "Warehouse" , "Entrept", "Lagerhaus" ] ,
	      "waterfall" : [ "Dam" , "Barrage", "Damm" ] ,
	      "westoilfield" : [ "West Oilfield" , "Champ ptrolifre Ouest", "Westliches lfeld" ] ,
	      "westperimeter" : [ "West Perimeter" , "Ouest du primtre", "Randstellung West" ] ,
	      "wmineentrance" : [ "West Mine Entrance" , "Entre Ouest de la mine", "Eingang zur westlichen Mine" ] ,
	      "woodyard" : [ "Wood Yard" , "Parc  bois", "Holzlagerplatz" ],
	      "asylum" : [ "Asylum", " ", " " ], 
	      "entrance" : [ "Entrance", " ", " " ],
	      "church" : [ "Church", " ", " " ], 
	      "uppervillage" : [ "Upper Village", " ", " " ],
      	      "suburbs" : [ "Suburbs", " ", " " ],
      	      "centralsquare" : [ "Central Square", " ", " " ],
      	      "convoy" : [ "Convoy", " ", " " ],
      	      "waterstation" : [ "South Bridge", " ", " " ],
	      "engine_room" : [ "Engine Room", " ", " " ], 
	      "bunks" : [ "Bunks", " ", " " ], 
	      "hanger_bay_fore" : [ "Hanger Bay Fore", " ", " " ], 
	      "mess_hall" : [ "Mess Hall", " ", " " ], 
	      "beach" : [ "Beach", " ", " " ], 
	      "supplypier" : [ "North Pier Guard House", " ", " " ], 
	      "commbuilding" : [ "Communications Building", " ", " " ], 
	      "shore" : [ "Shore", " ", " " ], 
	      "scrapyard" : [ "Scrap Yard", " ", " " ], 
	      "facility" : [ "Chemical Weapons Factory", " ", " " ], 
	      "yard" : [ "Train Yard", " ", " " ], 
	      "cargodepot" : [ "Cargo Depot", " ", " " ], 
	      "cargoplane" : [ "Cargo Plane", " ", " " ], 
	      "terminalcomplex" : [ "Airport Terminal", " ", " " ], 
	      "commandbunker" : [ "Commandbunker", " ", " " ], 
	      "launchpad" : [ "Launch Pad", " ", " " ], 
	      "palacegardens" : [ "Palace Gardens", " ", " " ], 
	      "vault" : [ "Palace, Third Floor", " ", " " ], 
	      "baseentrance" : [ "Base Entrance", " ", " " ], 
	      "securityoffice" : [ "Palace Security Office", " ", " " ],
	      "hilltop" : [ "Guard House", " ", " " ],
	      "uptown" : [ "Upper Village", " ", " " ],
	      "lowtown" : [ "Lower Village", " ", " " ],
	      "waterfront" : [ "North Bridge", " ", " " ],
	      "uppernorthside" : [ "Upper North Side", " ", " " ],
	      "uppersouthside" : [ "Upper South Side", " ", " " ],
	      "controltower" : [ "Control Tower", " ", " " ],
	      "welldeck" : [ "Well Deck", " ", " " ],
	      "hanger_bay_aft" : [ "Hanger Bay Aft", " ", " " ],
	      "controltowerexterior" : [ "Control Tower Exterior", " ", " " ],
	      "fueldepot" : [ "Fuel Depot", " ", " " ],
	      "supplydepot" : [ "Supply Depot", " ", " " ],
	      "supplypier_south" : [ "Supply Pier South", " ", " " ],
	      "subpenleft" : [ "Subpen Left", " ", " " ],
	      "subpenright" : [ "Subpen Right", " ", " " ],
	      "armory" : [ "Armory", " ", " " ],
	      "coolingstation" : [ "Cooling Station", " ", " " ],
	      "aircontroltower" : [ "Air Control Tower", " ", " " ],
	      "innerridge" : [ "Inner Ridge", " ", " " ],
	      "conferenceroom" : [ "Palace, Second Floor", " ", " " ],
	      "stationplaza" : [ "TV Station Plaza", " ", " " ],
	      "insurgenthideout" : [ "Insurgent Hideout", " ", " " ],
	      "parkinggarage" : [ "Plaza", " ", " " ],
	      "submarine" : [ "Submarine", " ", " " ],
	      "causeway" : [ "Causeway", " ", " " ],
	      "marketsquare" : [ "Market Square", " ", " " ],
	      "portfactory" : [ "Port Factory", " ", " " ],
	      "beachstreet" : [ "Beach Street", " ", " " ],
	      "oilplatform" : [ "Oil Platform", " ", " " ],
	      "oilfield" : [ "Oil Field", " ", " " ],
	      "subpen01" : [ "Sub Pen 1", " ", " " ],
	      "subpen02" : [ "Sub Pen 2", " ", " " ],
	      "supply_depot" : [ "Supply Depot", " ", " " ],
	      "fuel_depot" : [ "Fuel Depot", " ", " " ],
	      "helipad" : [ "Helipad", " ", " " ],
	      "outerridge" : [ "Outer Hill", " ", " " ],
	      "checkpoint" : [ "Checkpoint", " ", " " ],
	      "junctionpark" : [ "Junction Park", " ", " " ],
	      "hillcrest" : [ "Hill Crest", " ", " " ],
	      "hillcrest" : [ "Hill Crest", " ", " " ],
	      "oceanfront" : [ "Ocean Front", " ", " " ],
	      "overpass" : [ "Overpass", " ", " " ],
	      "mall" : [ "mall", " ", " " ],
	      "farmhouse" : [ "FarmHouse", " ", " " ],
	      "gasstation" : [ "Gas Station", " ", " " ],
	      "junkyard" : [ "Junk Yard", " ", " " ],
	      "overpass" : [ "Overpass", " ", " " ],
	      "truckstop" : [ "Truck Stop", " ", " " ],
	      "southernoildepot" : [ "Southern Oil Depot", " ", " " ],
	      "quantower" : [ "Quan Tower", " ", " " ],
	      "quantemple" : [ "Quan Temple", " ", " " ],
	      "fortressvillage" : [ "Fortress Village", " ", " " ],
	      "compoundmainvillage" : [ "Compound Main Village", " ", " " ],
	      "euairdrop" : [ "EU Strike Point", " ", " " ],	      
	      "QuanFortress" : [ "Quan Fortress", " ", " " ],
	      "compoundgatevillage" : [ "Compound Gate Village", " ", " " ],
	      "quancompound" : [ "Quan Compound", " ", " " ],
	      "huangvillage" : [ "Huang Village", " ", " " ],
	      "camyin" : [ "Cam Yin", " ", " " ],
	      "laosenghill" : [ "Lao Seng Hill", " ", " " ],
	      "laotianyeroad" : [ "Lao Tian Ye Road", " ", " " ],
	      "village" : [ "South Village", " ", " " ],
	      "oilplant" : [ "Oil Storage Facility", " ", " " ],
	      "garage" : [ "North Village", " ", " " ],
	      "oasis" : [ "Oasis Outpost", " ", " " ],
	      "refineryoutpost" : [ "Refinery Outpost", " ", " " ],
	      "turbinehill" : [ "Turbine Hill", " ", " " ],
	      "hillshed" : [ "Hill Shed", " ", " " ],
	      "checkpointbarndoor" : [ "Checkpoint Barndoor", " ", " " ],
	      "usforwardpost" : [ "US Forward Post", " ", " " ],
	      "grainelevator" : [ "Grain Elevator", " ", " " ],
	      "mecfieldbase" : [ "MEC Field Base", " ", " " ],
	      "dairyfarm" : [ "Dairy Farm", " ", " " ],
	      "appleciderfarm" : [ "Apple Cider Farm", " ", " " ],
	      "fieldequipmentbarn" : [ "Field Equipment Barn", " ", " " ],
	      "dairyfarm" : [ "Dairy Farm", " ", " " ],
	      "mecscoutpost" : [ "MEC Scout Post", " ", " " ],
	      "mecfieldcamp" : [ "MEC Field Camp", " ", " " ],
	      "armoredconvoy" : [ "Armored Convoy", " ", " " ],
	      "usstrikeopsbase" : [ "US Strike Operations Base", " ", " " ],
	      "hilltopoverlook" : [ "Hilltop Overlook", " ", " " ]

}

aasL10nTexts = [ [ "Flags must be captured in correct order!", "You have entered the ", " must be captured before the ", "Proceed to the " ] ,
		 [ "Capturez les points dans le bon ordre!", "Vous tes au point : ", " : doit tre captur avant : ", "Capturez : " ] ,
		 [ "Die Flaggen mssen in einer bestimmten Reihenfolge erobert werden!", "Gerade erobert: ", " muss zuerst erobert werden vor ", "Nchster Eroberungspunkt: " ] ]

aasL10nText = aasL10nTexts[aasL10n]



sdDebug = 0
sdLast = [ 0, 0, 0 ]
sdNextCp = [ 0, 0, 0 ]

sdL10nCp = { 	"aircontroltower" : "Radar Dish",
		"aircontroltower_chi" : "Radar Dish",
		"aircontroltower_mec" : "Radar Dish",
		"mobileradar_us_dest" : "UAV Trailer",
		"mobileradar_ch_dest" : "UAV Trailer",
		"mobileradar_mech_dest" : "UAV Trailer",
		"usart_lw155" : "Artillery",
		"ars_d30" : "Artillery"
}

# This array should contain the two base control points and all the objective names for each map.
# Format: Team1 base, Team2 base, Extra base1 (necessary on some maps), Extra base1, Team1 radar, Team1 UAV, Team1 Artillery #1, Team1 Artillery #2, Team2 radar, Team2 UAV, Team2 Artillery #1, Team2 Artillery #2 
def getBases():
	sdMaps = { "16" :
		    { 
		        "OperationHarvest" :
			[ "cp_16p_oh_mecfieldbase", "cp_16p_oh_checkpointbarndoor", "", "", "mobileradar_mech_dest", "aircontroltower_mec", "ARS_D30", "", "mobileradar_us_dest", "aircontroltower", "USART_LW155", "" ] ,
		        "OperationSmokeScreen" :
			[ "cp_16p_oss_oasis", "cp_16p_oss_village", "", "", "mobileradar_mech_dest", "aircontroltower_mec", "ARS_D30", "", "mobileradar_eu_dest", "aircontroltower_eu", "USART_LW155", "" ] ,
		        "GreatWall" :
			[ "cp_16_gw_compoundmainvillage", "cp_16_gw_euairdrop", "", "", "mobileradar_ch_dest", "aircontroltower_chi", "ARS_D30", "", "mobileradar_eu_dest", "aircontroltower_eu", "USART_LW155", "" ] ,
		        "OperationRoadRage" :
			[ "cp_16p_rr_mall", "cp_16p_rr_oceanfront", "", "", "mobileradar_mech_dest", "aircontroltower_mec", "ARS_D30", "", "mobileradar_us_dest", "aircontroltower", "USART_LW155", "" ] ,
		        "Dalian_plant" :
			[ "cpname_dp_16_powerplant", "cpname_dp_16_constructionsite", "", "", "mobileradar_ch_dest", "aircontroltower_chi", "ARS_D30", "", "mobileradar_us_dest", "aircontroltower", "USART_LW155", "" ] ,
			"Daqing_oilfields" :
			[ "cpname_do_16_oldcity", "cpname_do_16_refinery", "", "", "mobileradar_ch_dest", "aircontroltower_chi", "ARS_D30", "", "mobileradar_us_dest", "aircontroltower", "USART_LW155", "" ] ,
			"Dragon_Valley" :
			[ "cpname_dv_16_refinery", "cpname_dv_16_woodyard", "", "", "mobileradar_ch_dest", "aircontroltower_chi", "ars_d30", "", "mobileradar_us_dest", "aircontroltower", "usart_lw155", "" ] ,
			"FuShe_Pass" :
			[ "cpname_gp_16_wmineentrance", "cpname_gp_16_emineentrance", "", "", "mobileradar_ch_dest", "aircontroltower_chi", "ARS_D30", "", "mobileradar_us_dest", "aircontroltower", "USART_LW155", "" ] ,
			"Gulf_of_Oman" :
			[ "cpname_go_16_constructionsite", "cpname_go_16_beach", "", "", "mobileradar_mech_dest", "aircontroltower_mec", "ARS_D30", "", "mobileradar_us_dest", "aircontroltower", "USART_LW155", "" ] ,
			"kubra_dam" :
			[ "cpname_kd_16_powerstation", "cpname_kd_16_constructionsite", "", "", "mobileradar_mech_dest", "aircontroltower_mec", "ARS_D30", "", "mobileradar_us_dest", "aircontroltower", "USART_LW155", "" ] ,
			"Mashtuur_City" :
			[ "cpname_mc_16_hotel", "cpname_mc_16_southblock", "", "", "mobileradar_mech_dest", "aircontroltower_mec", "ARS_D30", "", "mobileradar_us_dest", "aircontroltower", "USART_LW155", "" ] ,
			"Operation_Clean_Sweep":
			[ "cpname_oc_16_smallairstrip", "cpname_oc_16_airfield", "", "", "mobileradar_mech_dest", "aircontroltower_mec", "ARS_D30", "", "mobileradar_us_dest", "aircontroltower", "USART_LW155", "" ] ,
			"Sharqi_Peninsula" :
			[ "cpname_sp_16_hotel", "cpname_sp_16_tvstation", "", "", "mobileradar_mech_dest", "aircontroltower_mec", "ARS_D30", "", "mobileradar_us_dest", "aircontroltower", "USART_LW155", "" ] ,
			"Songhua_Stalemate" :
			[ "cpname_ss_16_centralfront", "cpname_ss_16_westperimeter", "", "", "mobileradar_ch_dest", "aircontroltower_chi", "ARS_D30", "", "mobileradar_us_dest", "aircontroltower", "USART_LW155", "" ] ,
			"Strike_at_Karkand" :
			[ "cpname_sk_16_market", "cpname_sk_16_gasstation", "", "", "mobileradar_mech_dest", "aircontroltower_mec", "ARS_D30", "", "mobileradar_us_dest", "aircontroltower", "USART_LW155", "" ] ,
			"Zatar_Wetlands" :
			[ "cpname_zw_16_naturalgasstation", "cpname_zw_16_factory", "", "", "mobileradar_mech_dest", "aircontroltower_mec", "ARS_D30", "", "mobileradar_us_dest", "aircontroltower", "USART_LW155", "" ],
			"Devils_Perch" :
			[ "cp_16p_dp_asylum", "cp_16p_dp_villagecenter", "", "", "mobileradar_mech_dest", "aircontroltower_mec", "ars_d30", "", "mobileradar_us_dest", "aircontroltower", "USART_LW155", "" ] ,
			"Ghost_Town" :
			[ "cp_16p_gt_suburbs", "cp_16p_gt_waterstation", "", "", "mobileradar_mech_dest", "aircontroltower_mec", "ars_d30", "", "mobileradar_us_dest", "aircontroltower", "usart_lw155", "" ] ,
			"Iron_Gator" :
			[ "cp_16p_ms_engine_room", "cp_16p_ms_hanger_bay_fore", "", "", "mobileradar_mech_dest", "", "", "", "mobileradar_us_dest", "aircontroltower", "", "" ] ,
			"Leviathan" :
			[ "cp_16p_lev_insurgentwarehouse", "cp_16p_lev_commbuilding", "", "", "mobileradar_mech_dest", "", "", "", "mobileradar_us_dest", "aircontroltower", "usart_lw155", "" ] ,
			"Mass_Destruction" :
			[ "cp_16p_md_shore", "cp_16p_md_scrapyard", "", "", "mobileradar_mech_dest", "aircontroltower_mec", "ars_d30", "", "mobileradar_mech_dest", "aircontroltower_mec", "", "" ] ,
			"Night_Flight" :
			[ "cp_16p_nf_cargodepot", "cp_16p_nf_helipad", "", "", "mobileradar_mech_dest", "aircontroltower_mec", "ars_d30", "", "mobileradar_us_dest", "aircontroltower", "usart_lw155", "" ] ,
			"Surge" :
			[ "cp_16p_surge_commandbunker", "cp_16p_surge_baseentrance", "", "", "mobileradar_mech_dest", "aircontroltower_mec", "ARS_D30", "", "mobileradar_mech_dest", "aircontroltower_mec", "ars_d30", "" ] ,
			"Warlord" :
			[ "cp_16p_wl_conferenceroom", "cp_16p_wl_sasbase", "", "", "mobileradar_mech_dest", "aircontroltower_mec", "ARS_D30", "", "mobileradar_us_dest", "aircontroltower", "usart_lw155", "" ] } ,
			"32" :
			{ 
		        "OperationHarvest" :
			[ "cp_32p_oh_mecfieldcamp", "cp_32p_oh_appleciderfarm", "", "", "mobileradar_mech_dest", "aircontroltower_mec", "ARS_D30", "", "mobileradar_us_dest", "aircontroltower", "USART_LW155", "" ] ,
		        "OperationSmokeScreen" :
			[ "cp_32p_oss_oasis", "cp_32p_oss_garage", "", "", "mobileradar_mech_dest", "aircontroltower_mec", "ARS_D30", "", "mobileradar_eu_dest", "aircontroltower_eu", "USART_LW155", "" ] ,
		        "GreatWall" :
			[ "cp_32p_rr_mall", "cp_32p_rr_oceanfront", "", "", "mobileradar_ch_dest", "aircontroltower_ch", "ARS_D30", "", "mobileradar_eu_dest", "aircontroltower_eu", "USART_LW155", "" ] ,
		        "OperationRoadRage" :
			[ "cp_32p_rr_southernoildepot", "cp_32p_rr_farmhouse", "", "", "mobileradar_mech_dest", "aircontroltower_mec", "ARS_D30", "", "mobileradar_us_dest", "aircontroltower", "USART_LW155", "" ] ,
			"Dalian_plant" :
			[ "cpname_dp_32_airfield", "cpname_dp_32_uscarrier", "", "", "mobileradar_ch_dest", "aircontroltower_chi", "", "", "mobileradar_us_dest", "aircontroltower", "", "" ] ,
			"Daqing_oilfields" :
			[ "cpname_do_32_ustown", "cpname_do_32_chinesebase", "", "", "mobileradar_ch_dest", "aircontroltower_chi", "ARS_D30", "", "mobileradar_us_dest", "aircontroltower", "USART_LW155", "" ] ,
			"Dragon_Valley" :
			[ "cpname_dv_32_refinery", "cpname_dv_32_marketplace", "", "", "mobileradar_ch_dest", "aircontroltower_chi", "ars_d30", "", "mobileradar_us_dest", "aircontroltower", "usart_lw155", "" ] ,
			"FuShe_Pass" :
			[ "cpname_gp_32_uppercamp", "cpname_gp_32_powerplant", "", "", "mobileradar_ch_dest", "aircontroltower_chi", "ARS_D30", "", "mobileradar_us_dest", "aircontroltower", "USART_LW155", "" ] ,
			"Gulf_of_Oman" :
			[ "cpname_go_32_airfieldbase", "cpname_go_32_uscarrier", "", "", "mobileradar_mech_dest", "aircontroltower_mec", "ARS_D30", "", "mobileradar_us_dest", "aircontroltower", "", "" ] ,
			"kubra_dam" :
			[ "cpname_kd_32_intake", "cpname_kd_32_abandonedhuts", "", "", "mobileradar_mech_dest", "aircontroltower_mec", "ARS_D30", "", "mobileradar_us_dest", "aircontroltower", "USART_LW155", "" ] ,
			"Mashtuur_City" :
			[ "cpname_mc_32_hotel", "cpname_mc_32_hill", "", "", "mobileradar_mech_dest", "aircontroltower_mec", "ARS_D30", "", "mobileradar_us_dest", "aircontroltower", "USART_LW155", "" ] ,
			"Operation_Clean_Sweep":
			[ "cpname_oc_32_communicationcentral", "cpname_oc_32_airfield", "", "", "mobileradar_mech_dest", "aircontroltower_mec", "ARS_D30", "", "mobileradar_us_dest", "aircontroltower", "USART_LW155", "" ] ,
			"Sharqi_Peninsula" :
			[ "cpname_sp_32_beachhouse", "cpname_sp_32_tvstation", "", "", "mobileradar_mech_dest", "aircontroltower_mec", "ARS_D30", "", "mobileradar_us_dest", "aircontroltower", "USART_LW155", "" ] ,
			"Songhua_Stalemate" :
			[ "cpname_ss_32_chinesehq", "cpname_ss_32_fishfactory", "", "", "mobileradar_ch_dest", "aircontroltower_chi", "ARS_D30", "", "mobileradar_us_dest", "aircontroltower", "USART_LW155", "" ] ,
			"Strike_at_Karkand" :
			[ "cpname_sk_32_gatehouse", "cpname_sk_32_gasstation", "", "", "mobileradar_mech_dest", "aircontroltower_mec", "ARS_D30", "", "mobileradar_us_dest", "aircontroltower", "USART_LW155", "" ] ,
			"Zatar_Wetlands" :
			[ "cpname_zw_32_mecbase", "cpname_zw_32_airfield", "", "", "mobileradar_mech_dest", "aircontroltower_mec", "ARS_D30", "", "mobileradar_us_dest", "aircontroltower", "USART_LW155", "" ],
			"Devils_Perch" :
			[ "cp_32p_dp_asylum", "cp_32p_dp_submarine", "", "", "mobileradar_mech_dest", "aircontroltower_mec", "ARS_D30", "", "mobileradar_us_dest", "aircontroltower", "USART_LW155", "" ] ,
			"Ghost_Town" :
			[ "cp_32p_gt_resort", "cp_32p_gt_dam", "", "", "mobileradar_mech_dest", "aircontroltower_mec", "ARS_D30", "", "mobileradar_us_dest", "aircontroltower", "usart_lw155", "" ] ,
			"Iron_Gator" :
			[ "cp_32p_ms_controltower", "cp_32p_ms_hanger_bay_fore", "", "", "mobileradar_mech_dest", "aircontroltower_mec", "", "", "mobileradar_us_dest", "aircontroltower", "", "" ] , 
			"Leviathan" :
			[ "cp_32p_lev_fueldepot", "cp_32p_lev_armory", "", "", "mobileradar_mech_dest", "", "ARS_D30", "", "mobileradar_us_dest", "aircontroltower", "usart_lw155", "" ] ,
			"Mass_Destruction" :
			[ "cp_32p_md_island", "cp_32p_md_facility", "", "", "mobileradar_mech_dest", "aircontroltower_mec", "ars_d30", "", "mobileradar_mech_dest", "aircontroltower_mec", "ars_d30", "" ] ,
			"Night_Flight" :
			[ "cp_32p_nf_aircontroltower", "cp_32p_nf_helipad", "", "", "mobileradar_mech_dest", "aircontroltower_mec", "ars_d30", "", "mobileradar_us_dest", "aircontroltower", "usart_lw155", "" ] ,
			"Surge" :
			[ "cp_32p_surge_commandbunker", "cp_32p_surge_outerridge", "", "", "mobileradar_mech_dest", "aircontroltower_mec", "ARS_D30", "", "mobileradar_mech_dest", "aircontroltower", "usart_lw155", "" ] ,
			"Warlord" :
			[ "cp_32p_wl_conferenceroom", "cp_32p_wl_constructionsite", "", "", "mobileradar_mech_dest", "aircontroltower_mec", "ars_d30", "", "mobileradar_us_dest", "aircontroltower", "usart_lw155", "" ] } ,
			"64" :
			{ 
		        "OperationHarvest" :
			[ "cp_64p_oh_mecfieldbase", "cp_64p_oh_usstrikeopsbase", "", "", "mobileradar_mech_dest", "aircontroltower_mec", "ARS_D30", "", "mobileradar_us_dest", "aircontroltower", "USART_LW155", "" ] ,
			"OperationRoadRage" :
			[ "cp_64p_rr_southernoildepot", "cp_64p_rr_farmhouse", "", "", "mobileradar_mech_dest", "aircontroltower_mec", "ARS_D30", "", "mobileradar_us_dest", "aircontroltower", "USART_LW155", "" ] ,
			"Dalian_plant" :
			[ "cpname_dp_64_airfield", "cpename_dp_64_northdocks", "", "", "mobileradar_ch_dest", "aircontroltower_chi", "ARS_D30", "", "mobileradar_us_dest", "aircontroltower", "USART_LW155", "" ] ,
			"Daqing_oilfields" :
			[ "cpname_do_64_chinesebase", "cpname_do_64_ustown", "", "", "mobileradar_ch_dest", "aircontroltower_chi", "ARS_D30", "", "mobileradar_us_dest", "aircontroltower", "USART_LW155", "" ] ,
			"Dragon_Valley" :
			[ "cpname_dv_64_refinery", "cpname_dv_64_docks", "", "", "mobileradar_ch_dest", "aircontroltower_chi", "ars_d30", "", "mobileradar_us_dest", "aircontroltower", "usart_lw155", "" ] ,
			"FuShe_Pass" :
			[ "cpname_gp_64_chineseairfield", "cpname_gp_64_usairfield", "", "", "mobileradar_ch_dest", "aircontroltower_chi", "ARS_D30", "", "mobileradar_us_dest", "aircontroltower", "USART_LW155", "" ] ,
			"Gulf_of_Oman" :
			[ "cpname_go_64_airfieldbase", "cpname_go_64_uscarrier", "", "", "mobileradar_mech_dest", "aircontroltower_mec", "ARS_D30", "", "mobileradar_us_dest", "aircontroltower", "", "" ] ,
			"kubra_dam" :
			[ "cpname_kd_64_intake", "cpname_kd_64_abandonedhuts", "", "", "mobileradar_mech_dest", "aircontroltower_mec", "ARS_D30", "", "mobileradar_us_dest", "aircontroltower", "USART_LW155", "" ] ,
			"Mashtuur_City" :
			[ "cpname_mc_64_ngasstation", "cpname_mc_64_sgasstation", "", "", "mobileradar_mech_dest", "aircontroltower_mec", "ARS_D30", "", "mobileradar_us_dest", "aircontroltower", "USART_LW155", "" ] ,
			"Operation_Clean_Sweep":
			[ "cpname_oc_64_mecairfield", "cpname_oc_64_airfield", "", "", "mobileradar_mech_dest", "aircontroltower_mec", "ARS_D30", "", "mobileradar_us_dest", "aircontroltower", "USART_LW155", "" ] ,
			"Sharqi_Peninsula" :
			[ "cpname_sp_64_beachhouse", "cpname_sp_64_tvstation", "", "", "mobileradar_mech_dest", "aircontroltower_mec", "ARS_D30", "", "mobileradar_us_dest", "aircontroltower", "USART_LW155", "" ] ,
			"Songhua_Stalemate" :
			[ "cpname_ss_64_chinesehq", "cpname_ss_64_fishfactory", "", "", "mobileradar_ch_dest", "aircontroltower_chi", "ARS_D30", "", "mobileradar_us_dest", "aircontroltower", "USART_LW155", "" ] ,
			"Strike_at_Karkand" :
			[ "cpname_sk_64_factory", "cpname_sk_64_gasstation", "", "", "mobileradar_mech_dest", "aircontroltower_mec", "ARS_D30", "", "mobileradar_us_dest", "aircontroltower", "USART_LW155", "" ] ,
			"Zatar_Wetlands" :
			[ "cpname_zw_64_mecbase", "cpname_zw_64_beach", "", "", "mobileradar_mech_dest", "aircontroltower_mec", "ARS_D30", "", "mobileradar_us_dest", "aircontroltower", "USART_LW155", "" ],
			"Devils_Perch" :
			[ "cp_64p_dp_asylum", "cp_64p_dp_submarine", "", "", "mobileradar_mech_dest", "aircontroltower_mec", "ARS_D30", "", "mobileradar_us_dest", "aircontroltower", "usart_lw155", "" ] ,
			"Ghost_Town" :
			[ "cp_64p_gt_resort", "cp_64p_gt_dam", "cp_64p_gt_oilplatform", "cp_64p_gt_oilfield", "mobileradar_mech_dest", "aircontroltower_mec", "ARS_D30", "", "mobileradar_us_dest", "aircontroltower", "usart_lw155", "" ] ,
			"Iron_Gator" :
			[ "cp_64p_ms_insurgentbase", "cp_64p_ms_controltower", "cp_64p_ms_controltowerexterior", "", "mobileradar_mech_dest", "aircontroltower_mec", "", "", "mobileradar_us_dest", "aircontroltower", "", "" ] ,
			"Leviathan" :
			[ "cp_64p_lev_insurgent_warehouse", "", "", "cp_64p_lev_armory", "mobileradar_mech_dest", "aircontroltower_mec", "ars_d30", "", "mobileradar_us_dest", "aircontroltower", "usart_lw155", "" ] ,
			"Mass_Destruction" :
			[ "cp_64p_md_dumpsite", "cp_64p_md_facility", "", "", "mobileradar_mech_dest", "aircontroltower_mec", "ars_d30", "", "mobileradar_mech_dest", "aircontroltower_mec", "ars_d30", "" ] ,
			"Night_Flight" :
			[ "cp_64p_nf_aircontroltower", "cp_64p_nf_gate", "", "", "mobileradar_mech_dest", "aircontroltower_mec", "ars_d30", "", "mobileradar_us_dest", "aircontroltower", "usart_lw155", "" ] ,
			"Surge" :
			[ "cp_64p_surge_commandbunker", "cp_64p_surge_village", "", "", "mobileradar_mech_dest", "aircontroltower_mec", "ARS_D30", "", "mobileradar_mech_dest", "aircontroltower", "usart_lw155", "" ] ,
			"Warlord" :
			[ "cp_64p_wl_conferenceroom", "cp_64p_wl_constructionsite", "", "", "mobileradar_mech_dest", "aircontroltower_mec", "ars_d30", "", "mobileradar_us_dest", "aircontroltower", "usart_lw155", "" ] } }
		    
	sdCps = False
	for obj in bf2.objectManager.getObjectsOfType('dice.hfe.world.ObjectTemplate.ControlPoint'):
		obj = obj.templateName

		# Extract the map size from the template name.
		sdMapSize = obj[10:12]

		# Vanilla BF2 has a different CP naming sceme than SF, so this needs to be taken into account.
		if sdMapSize != "16" and sdMapSize != "32" and sdMapSize != "64":
			sdMapSize = obj[3:5]

		# Walke island is a different animal than the rest.
		if sdMapSize != "16" and sdMapSize != "32" and sdMapSize != "64":
			sdMapSize = obj[0:2]

		break
	sdMapsOfSize = sdMaps[sdMapSize]
	sdCps = sdMapsOfSize[bf2.gameLogic.getMapName()]

	return sdCps

# See if the given control point is a base.
def isBase(cp):
	sdCps = getBases()
	if sdCps:
		if sdDebug == 1:
			host.rcon_invoke('game.sayall yo!' + sdCps[0])

		if cp.templateName.lower() == sdCps[0] or cp.templateName.lower() == sdCps[1] or cp.templateName.lower() == sdCps[2] or cp.templateName.lower() == sdCps[2]:
			return False
		else:
			return True

def aasSetNextCp(aasTeam, aasInit):
	aasMaps = { "16" :
		    { 
			"OperationHarvest" :
			[ "cp_16p_oh_grainelevator", "cp_16p_oh_dairyfarm", "cp_16p_usforwardpost", "cp_16p_oh_checkpointbarndoor"] ,
			"OperationSmokeScreen" :
			[ "cp_16p_oss_village", "cp_16p_oss_oilplant", "cp_16p_oss_garage", "cp_16p_oss_oasis"] ,
			"GreatWall" :
			[ "cp_16_gw_compoundmainvillage", "cp_16_gw_compoundgatevillage", "cp_16_gw_quantower", "cp_16_gw_quantemple", "cp_16_gw_fortressvillage"] ,
		        "OperationRoadRage" :
			[ "cp_16p_rr_overpass", "cp_16p_rr_junctionpark", "cp_16p_rr_hillcrest" ] ,
		        "Dalian_plant" :
			[ "cpname_dp_16_powerplant", "cpname_dp_16_mainentrance", "cpname_dp_16_constructionsite", "cpname_dp_16_reactors" ] ,
			"Daqing_oilfields" :
			[ "cpname_do_16_oldcity", "cpname_do_16_oilcisterns", "cpname_do_16_refinery" ] ,
			"Dragon_Valley" :
			[ "cpname_dv_16_refinery", "cpname_dv_16_powerstation", "cpname_dv_16_hillvillage" , "cpname_dv_16_woodyard" ] ,
			"FuShe_Pass" :
			[ "cpname_gp_16_wmineentrance", "cpname_gp_16_island", "cpname_gp_16_emineentrance" ] ,
			"Gulf_of_Oman" :
			[ "cpname_go_16_constructionsite", "cpname_go_16_city", "cpname_go_16_village", "cpname_go_16_beach" ] ,
			"kubra_dam" :
			[ "cpname_kd_16_powerstation", "cpname_kd_16_lowerdam", "cpname_kd_16_upperdam", "cpname_kd_16_constructionsite" ] ,
			"Mashtuur_City" :
			[ "cpname_mc_16_hotel", "cpname_mc_16_northbridge", "cpname_mc_16_roadconstruct", "cpname_mc_16_southblock" ] ,
			"Operation_Clean_Sweep":
			[ "cpname_oc_16_smallairstrip", "cpname_oc_16_refuellingstation", "cpname_oc_16_controlcenter", "cpname_oc_16_airfield" ] ,
			"Sharqi_Peninsula" :
			[ "cpname_sp_16_hotel", "cpname_sp_16_constructionsite", "cpname_sp_16_officebuildings", "cpname_sp_16_tvstation" ] ,
			"Songhua_Stalemate" :
			[ "cpname_ss_16_centralfront", "cpname_ss_16_northisland", "cpname_ss_16_temple", "cpname_ss_16_westperimeter" ] ,
			"Strike_at_Karkand" :
			[ "cpname_sk_16_market", "cpname_sk_16_square", "cpname_sk_16_hotel", "cpname_sk_16_gasstation" ] ,
			"Zatar_Wetlands" :
			[ "cpname_zw_16_naturalgasstation", "cpname_zw_16_centralisland", "cpname_zw_16_factory" ],		    
			"Devils_Perch" :
			[ "cp_16p_dp_asylum", "cp_16p_dp_gated_entrance", "cp_16p_dp_church", "cp_16p_dp_uppervillage" ] ,
			"Ghost_Town" :
			[ "cp_16p_gt_suburbs", "cp_16p_gt_centralsquare", "cp_16p_gt_convoy", "cp_16p_gt_waterstation" ] ,
			"Iron_Gator" :
			[ "cp_16p_ms_engine_room", "cp_16p_ms_bunks", "cp_16p_ms_hanger_bay_fore", "cp_16p_ms_mess_hall" ] ,
			"Leviathan" :
			[ "cp_16p_lev_beach", "cp_16p_lev_powerstation", "cp_16p_lev_supplypier", "cp_16p_lev_commbuilding" ] ,
			"Mass_Destruction" :
			[ "cp_16p_md_shore", "cp_16p_md_facility", "cp_16p_md_yard", "cp_16p_md_scrapyard" ] ,
			"Night_Flight" :
			[ "cp_16p_nf_cargodepot", "cp_16p_nf_cargoplane", "cp_16p_nf_terminalcomplex" ] ,
			"Surge" :
			[ "cp_16p_surge_commandbunker", "cp_16p_surge_launchpad", "cp_16p_surge_baseentrance" ] ,
			"Warlord" :
			[ "cp_16p_wl_palacegardens", "cp_16p_wl_vault", "cp_16p_wl_conferenceroom", "cp_16p_wl_securityoffice" ] } ,
		"32" :
		     { 
			"OperationHarvest" :
			[ "cp_32p_oh_dairyfarm", "cp_32p_oh_grainelevator", "cp_32p_oh_mecscoutpost", "cp_32p_oh_checkpointbarndoor", "cp_32p_oh_fieldequipmentbarn", "cp_32p_oh_largeproducefarm" ] ,
			"OperationSmokeScreen" :
			[ "cp_32p_oss_garage", "cp_32p_oss_oilplant", "cp_32p_oss_hillshed", "cp_32p_oss_turbinehill", "cp_32p_oss_village"] ,
			"GreatWall" :
			[ "cp_32_gw_laosenghill", "cp_32_gw_quanfortress", "cp_32_gw_quancompound", "cp_32_gw_laotianyeroad"] ,
			"OperationRoadRage" :
			[ "cp_32p_rr_truckstop", "cp_32p_rr_mall", "cp_32p_rr_overpass", "cp_32p_rr_junkyard", "cp_32p_rr_gasstation" ] ,
			"Dalian_plant" :
			[ "cpname_dp_32_airfield", "cpname_dp_32_mainentrance", "cpname_dp_32_maintenancebuilding", "cpname_dp_32_southdocks", "cpname_dp_32_reactors", "cpname_dp_32_uscarrier" ] ,
			"Daqing_oilfields" :
			[ "cpname_do_32_chinesebase", "cpname_do_32_oldcity", "cpname_do_32_powerstation", "cpname_do_32_oilcisterns", "cpname_do_32_gasstation", "cpname_do_32_refinery", "cpname_do_32_ustown"] ,
			"Dragon_Valley" :
			[ "cpname_dv_32_refinery", "cpname_dv_32_powerstation", "cpname_dv_32_hillvillage", "cpname_dv_32_woodyard", "cpname_dv_32_islandfarmhouse", "cpname_dv_32_marketplace"] ,
			"FuShe_Pass" :
			[ "cpname_gp_32_uppercamp", "cpname_gp_32_bridgecamp", "cpname_gp_32_wmineentrance", "cpname_gp_32_emineentrance", "cpname_gp_32_securityhq", "cpname_gp_32_powerplant" ] ,
			"Gulf_of_Oman" :
			[ "cpname_go_32_airfieldbase", "cpname_go_32_constructionsite", "cpname_go_32_hotel", "cpname_go_32_rockfort", "cpname_go_32_olivefort", "cpname_go_32_riverfort", "cpname_go_32_uscarrier" ] ,
			"kubra_dam" :
			[ "cpname_kd_32_intake", "cpname_kd_32_constructionsite", "cpname_kd_32_powerstation", "cpname_kd_32_lowerdam", "cpname_kd_32_materialstation", "cpname_kd_32_refuellingstation", "cpname_kd_32_bridgebase", "cpname_kd_32_abandonedhuts" ] ,
			"Mashtuur_City" :
			[ "cpname_mc_32_hotel", "cpname_mc_32_powerplant", "cpname_mc_32_mosque", "cpname_mc_32_backyard", "cpname_mc_32_hill" ] ,
			"Operation_Clean_Sweep" :
			[ "cpname_oc_32_communicationcentral", "cpname_oc_32_abandonedvillage", "cpname_oc_32_smallairstrip", "cpname_oc_32_refuellingstation", "cpname_oc_32_controlcenter", "cpname_oc_32_airfield" ] ,
			"Sharqi_Peninsula" :
			[ "cpname_sp_32_beachhouse", "cpname_sp_32_clubhouse", "cpname_sp_32_outskirts", "cpname_sp_32_city_entrance", "cpname_sp_32_surveillancepost", "cpname_sp_32_hotel", "cpname_sp_32_constructionsite", "cpname_sp_32_tvstation" ] ,
			"Songhua_Stalemate" :
			[ "cpname_ss_32_chinesehq", "cpname_ss_32_centralfront", "cpname_ss_32_northisland", "cpname_ss_32_southisland", "cpname_ss_32_westperimeter", "cpname_ss_32_fishfactory" ] ,
			"Strike_at_Karkand" :
			[ "cpname_sk_32_gatehouse", "cpname_sk_32_trainaccident", "cpname_sk_32_market", "cpname_sk_32_suburb", "cpname_sk_32_square", "cpname_sk_32_hotel", "cpname_sk_32_gasstation" ] ,
			"Zatar_Wetlands" :
			[ "cpname_zw_32_mecbase", "cpname_zw_32_farm", "cpname_zw_32_naturalgasstation", "cpname_zw_32_centralisland", "cpname_zw_32_factory", "cpname_zw_32_village", "cpname_zw_32_airfield" ],
			"Devils_Perch" :
			[ "cp_32p_dp_asylum", "cp_32p_dp_church", "cp_32p_dp_hilltop", "cp_32p_dp_uptown", "cp_32p_dp_lowtown" ] ,
			"Ghost_Town" :
			[ "cp_32p_gt_waterfront", "cp_32p_gt_uppernorthside", "cp_32p_gt_uppersouthside", "cp_32p_gt_convoy", "cp_32p_gt_waterstation" ] ,
			"Iron_Gator" :
			[ "cp_32p_ms_controltower", "cp_32p_ms_welldeck", "cp_32p_ms_hanger_bay_aft", "cp_32p_ms_controltowerexterior", "cp_32p_ms_mess_hall", "cp_32p_ms_hanger_bay_fore" ] , 
			"Leviathan" :
			[ "cp_32p_lev_fueldepot", "cp_32p_lev_supplydepot", "cp_32p_lev_supplypier_south", "cp_32p_lev_subpenleft", "cp_32p_lev_subpenright", "cp_32p_lev_armory"] ,
			"Mass_Destruction" :
			[ "cp_32p_md_scrapyard", "cp_32p_md_coolingstation", "cp_32p_md_yard", "cp_32p_md_facility", "cp_32p_md_shore" ] ,
			"Night_Flight" :
			[ "cp_32p_nf_aircontroltower", "cp_32p_nf_cargodepot", "cp_32p_nf_cargoplane", "cp_32p_nf_terminalcomplex" ] ,
			"Surge" :
			[ "cp_32_surge_supplydepot", "cp_32p_surge_commandbunker", "cp_32p_surge_launchpad", "cp_32p_surge_baseentrance", "cp_32p_surge_innerridge" ] ,
			"Warlord" :
			[ "cp_32p_wl_vault", "cp_32p_wl_conferenceroom", "cp_32p_wl_stationplaza", "cp_32p_wl_securityoffice", "cp_32p_wl_insurgenthideout", "cp_32p_wl_stationplaza",  "cp_32p_wl_parkinggarage"] } ,
		"64" :
		     {
			"OperationHarvest" :
			[ "cp_64p_oh_mecfieldbase", "cp_64p_oh_dairyfarm", "cp_64p_oh_grainelevator", "cp_64p_oh_mecscoutpost", "cp_64p_oh_checkpointbarndoor", "cp_64p_oh_hilltopoverlook", "cp_64p_oh_armoredconvoy", "cp_64p_oh_largeproducefarm" ] ,
			"OperationRoadRage" :
			[ "cp_64p_rr_truckstop", "cp_64p_rr_mall", "cp_64p_rr_overpass", "cp_64p_rr_junkyard", "cp_64p_rr_gasstation" ] ,
			"Dalian_plant" :
			[ "cpname_dp_64_mainentrance", "cpname_dp_64_maintenancebuilding", "cpname_dp_64_southdocks", "cpname_dp_64_reactors", "cpname_dp_64_warehouse", "cpename_dp_64_northdocks" ] ,
			"Daqing_oilfields" :
			[ "cpname_do_64_powerstation", "cpname_do_64_westoilfield", "cpname_do_64_refinery", "cpname_do_64_oilcisterns", "cpname_do_64_oldcity", "cpname_do_64_eastoilfield", "cpname_do_64_gasstation", "cpname_do_64_ustown" ] ,
			"Dragon_Valley" :
			[ "cpname_dv_64_refinery", "cpname_dv_64_powerstation", "cpname_dv_64_hillvillage", "cpname_dv_64_woodyard", "cpname_dv_64_islandfarmhouse", "cpname_dv_64_marketplace", "cpname_dv_64_temple", "cpname_dv_64_rivervillage", "cpname_dv_64_vistapoint", "cpname_dv_64_docks" ] ,
			"FuShe_Pass" :
			[ "cpname_gp_64_chineseairfield", "cpname_gp_64_canyonguardpost", "cpname_gp_64_uppercamp", "cpname_gp_64_bridgecamp", "cpname_gp_64_wmineentrance", "cpname_gp_64_securityhq", "cpname_gp_64_powerplant", "cpname_gp_64_emineentrance", "cpname_gp_64_mountainlookout", "cpname_gp_64_usairfield" ] ,
			"Gulf_of_Oman" :
			[ "cpname_go_64_airfieldbase", "cpname_go_64_village", "cpname_go_64_constructionsite", "cpname_go_64_hotel", "cpname_go_64_villagefort", "cpname_go_64_rockfort", "cpname_go_64_olivefort", "cpname_go_64_riverfort", "cpname_go_64_uscarrier" ] ,
			"kubra_dam" :
			[ "cpname_kd_64_intake", "cpname_kd_64_constructionsite", "cpname_kd_64_powerstation", "cpname_kd_64_lowerdam", "cpname_kd_32_refuellingstation", "cpname_kd_64_materialstation", "cpname_kd_64_supervisorbase", "cpname_kd_64_bridgebase", "cpname_kd_64_abandonedhuts" ] ,
			"Mashtuur_City" :
			[ "cpname_mc_64_ngasstation", "cpname_mc_64_hotel", "cpname_mc_64_powerplant", "cpname_mc_64_mosque", "cpname_mc_64_backyard", "cpname_mc_64_hill", "cpname_mc_64_sgasstation" ] ,
			"Operation_Clean_Sweep" :
			[ "cpname_oc_64_mecairfield", "cpname_oc_64_harbor", "cpname_oc_64_communicationcentral", "cpname_oc_64_abandonedvillage", "cpname_oc_64_smallairstrip", "cpname_oc_64_refuellingstation", "cpname_oc_64_controlcenter", "cpname_oc_64_airfield" ] ,
			"Sharqi_Peninsula" :
			[ "cpname_sp_64_beachhouse", "cpname_sp_64_clubhouse", "cpname_sp_64_outskirts", "cpname_sp_64_city_entrance", "cpname_sp_64_surveillancepost", "cpname_sp_64_alley", "cpname_sp_64_constructionsite", "cpname_sp_64_hotel", "cpname_sp_64_tvstation" ] ,
			"Songhua_Stalemate" :
			[ "cpname_ss_64_chinesehq", "cpname_ss_64_centralfront", "cpname_ss_64_southfront", "cpname_ss_64_southisland", "cpname_ss_64_northisland", "cpname_ss_64_westperimeter", "cpname_ss_64_fishfactory", "cpname_ss_64_northperimeter" ] ,
			"Strike_at_Karkand" :
			[ "cpname_sk_64_factory", "cpname_sk_64_warehouse", "cpname_sk_64_cementfactory", "cpname_sk_64_gatehouse", "cpname_sk_64_trainaccident", "cpname_sk_64_suburb", "cpname_sk_64_square", "cpname_sk_64_hotel", "cpname_sk_64_gasstation" ] ,
			"Zatar_Wetlands" :
			[ "cpname_zw_64_mecbase", "cpname_zw_64_farm", "cpname_zw_64_naturalgasstation", "cpname_zw_64_centralisland", "cpname_zw_64_factory", "cpname_zw_64_village", "cpname_zw_64_airfield", "cpname_zw_64_beach", "cpname_zw_64_uscarrier" ],
			"Wake_Island_2007" :
			[ "64_ch_cp_southbase", "64_ch_cp_beach", "64_ch_cp_airfield", "64_ch_cp_northbase", "64_ch_cp_northvillage",  "64_us_cp_carrier" ],
			"Devils_Perch" :
			[ "cp_64p_dp_submarine", "cp_64p_dp_church", "cp_64p_dp_hilltop", "cp_64p_dp_uptown", "cp_64p_dp_lowtown", "cp_64p_dp_powerstation", "cp_64p_dp_causeway", "cp_64p_dp_marketsquare", "cp_64p_dp_portfactory" ] ,
			"Ghost_Town" :
			[ "cp_64p_gt_waterfront", "cp_64p_gt_uppernorthside", "cp_64p_gt_uppersouthside", "cp_64p_gt_convoy", "cp_64p_gt_waterstation", "cp_64p_gt_beachstreet", "cp_64p_gt_oilplatform", "cp_64p_gt_oilfield" ] ,
			"Iron_Gator" :
			[ "cp_64p_ms_controltower", "cp_64p_ms_welldeck", "cp_64p_ms_hanger_bay_aft", "cp_64p_ms_controltowerexterior", "cp_64p_ms_mess_hall", "cp_64p_ms_hanger_bay_fore" ] ,
			"Leviathan" :
			[ "cp_64p_lev_beach", "cp_64p_lev_powerstation", "cp_64p_lev_commbuilding", "cp_64p_lev_subpen02", "cp_64p_lev_subpen01", "cp_642p_lev_armory",  "cp_64p_lev_supply_depot", "cp_64p_lev_fuel_depot"] ,
			"Mass_Destruction" :
			[ "cp_64p_md_island", "cp_64p_md_scrapyard", "cp_64p_md_coolingstation", "cp_64p_md_yard", "cp_64p_md_facility", "cp_64p_md_shore" ] ,
			"Night_Flight" :
			[ "cp_64p_nf_aircontroltower", "cp_64p_nf_cargodepot", "cp_64p_nf_cargpolane", "cp_64p_nf_terminalcomplex", "cp_64p_nf_helipad" ] ,
			"Surge" :
			[ "cp_64p_surge_commandbunker", "cp_64p_surge_fueldepot", "cp_64p_surge_launchpad", "cp_64p_surge_supplydepot", "cp_64p_surge_innerridge", "cp_64p_surge_outerridge", "cp_64p_surge_checkpoint", "cp_64p_surge_outerridge" ] ,
			"Warlord" :
			[ "cp_64p_wl_vault", "cp_64p_wl_conferenceroom", "cp_64p_wl_securityoffice", "cp_64p_wl_mosque", "cp_64p_wl_insurgenthideout", "cp_64p_wl_stationplaza", "cp_64p_wl_parkinggarage" ] } }
		    
	aasCps = False
	for obj in bf2.objectManager.getObjectsOfType('dice.hfe.world.ObjectTemplate.ControlPoint'):
		obj = obj.templateName

		# Extract the map size from the template name.
		aasMapSize = obj[10:12]

		# Vanilla BF2 has a different CP naming sceme than SF, so this needs to be taken into account.
		if aasMapSize != "16" and aasMapSize != "32" and aasMapSize != "64":
			aasMapSize = obj[3:5]

		# Walke island is a different animal than the rest.
		if aasMapSize != "16" and aasMapSize != "32" and aasMapSize != "64":
			aasMapSize = obj[0:2]

		break
	aasMapsOfSize = aasMaps[aasMapSize]
	aasCps = aasMapsOfSize[bf2.gameLogic.getMapName()]

	aasTeam2 = 2
	if aasCps:
		if aasTeam == 2:
			aasTeam2 = 1
			aasCps.reverse()
		for aasCnt in range(0,len(aasCps)):
			aasNumOfCps = 0
			aasTeamCps = 0
			for obj in bf2.objectManager.getObjectsOfType('dice.hfe.world.ObjectTemplate.ControlPoint'):
				aasNumOfCps += 1
				if obj.cp_getParam('team') == aasTeam:
					aasTeamCps += 1
				if obj.templateName.lower() == aasCps[aasCnt]:
					aasCpId = obj
				elif obj.templateName.lower() == aasCps[aasCnt-1]:
					aasPrevCpId = obj
			if aasInit:
				if aasTeamCps < int(aasNumOfCps / 2):
					aasCpId.cp_setParam('flag', aasTeam)
					aasCpId.cp_setParam('team', aasTeam)
					aasCpId.cp_setParam('takeOverChangePerSecond', 100)
			elif aasCpId.cp_getParam('team') != aasTeam:
				if aasCnt == 0 or aasPrevCpId.cp_getParam('team') == aasTeam:
					aasNextCp[aasTeam] = aasCps[aasCnt]
					break


# Converts a template name into a readable text description.					
def aasCpName(aasCp):
	aasTempCp = aasCp.lower()

	aasList = aasTempCp.split('_')

	aasTempTranslations = aasL10nCp[aasList[3]]
	aasTempCp = aasTempTranslations[aasL10n]

	# If a properly worded control point can't be found, we should at the very least use the original template.
	if aasTempCp != "":
		aasCp = aasTempCp

	return aasCp


def init():
	global gameMode

	# events hook
	host.registerGameStatusHandler(onGameStatusChanged)
	if host.sgl_getIsAIGame() == 1:
		host.sh_setEnableCommander(aasCommander)
	else:
		host.sh_setEnableCommander(aasCommander)
		
	host.registerHandler('TimeLimitReached', onTimeLimitReached, 1)	

	if initialGameMode == "both":
		gameMode = "aas"
	else:
		gameMode = initialGameMode 

	if g_debug: print "gpm_cq.py initialized"
		
		
		
def deinit():
	bf2.triggerManager.destroyAllTriggers()
	global g_controlPoints
	g_controlPoints = []
	host.unregisterGameStatusHandler(onGameStatusChanged)
	
	if g_debug: print "gpm_cq.py uninitialized"



def onGameStatusChanged(status):
	global g_controlPoints
	global sdTeam1Objectives
	global sdTeam2Objectives
	global sdNumTeam1Objectives
	global sdNumTeam2Objectives
	global gameMode
	global sdAttackingTeam
	global aasWinner

	if status == bf2.GameStatus.Playing: 

		if gameMode == "aas":
			if aasCpBalance:
				aasSetNextCp(1, 1)
				aasSetNextCp(2, 1)

			# add control point triggers
			g_controlPoints = bf2.objectManager.getObjectsOfType('dice.hfe.world.ObjectTemplate.ControlPoint')
			for obj in g_controlPoints:
				radius = float(obj.getTemplateProperty('radius'))
				isHemi = int(obj.cp_getParam('isHemisphere'))

				if aasRadius > 0:
					aasRadiusFactor = float(host.ss_getParam('maxPlayers')) / aasRadius
					if aasRadiusFactor > 1:
						radius = round(radius * aasRadiusFactor)
				
				if isHemi != 0:
					id = bf2.triggerManager.createHemiSphericalTrigger(obj, onCPTrigger, '<<PCO>>', radius, (1, 2, 3))
				else:
					id = bf2.triggerManager.createRadiusTrigger(obj, onCPTrigger, '<<PCO>>', radius, (1, 2, 3))			
				obj.triggerId = id
				obj.lastAttackingTeam = 0
				if obj.cp_getParam('team') > 0:
					obj.flagPosition = Top
				else:
					obj.flagPosition = Bottom
		else:
			if sdAttackingTeam == 1:
				sdDefendingTeam = 2
			else:
				sdDefendingTeam = 1
				
			# Create objective arrays for each team.
			sdCps = getBases()
			sdTeam1Objectives = [ sdCps[4], sdCps[5], sdCps[6], sdCps[7] ]
			sdTeam2Objectives = [ sdCps[8], sdCps[9], sdCps[10], sdCps[11] ]

			# Get the number of valid objectives.
			sdNumTeam1Objectives = 0
			for cp in sdTeam1Objectives:
				if cp != "":
					sdNumTeam1Objectives = sdNumTeam1Objectives + 1

			sdNumTeam2Objectives = 0
			for cp in sdTeam2Objectives:
				if cp != "":
					sdNumTeam2Objectives = sdNumTeam2Objectives + 1

			# add control point triggers
			g_controlPoints = bf2.objectManager.getObjectsOfType('dice.hfe.world.ObjectTemplate.ControlPoint')
			for obj in g_controlPoints:
				# Set all capturably, non base flags to neutral.
				if obj.templateName.lower() != sdCps[0] and obj.templateName.lower() != sdCps[1] and obj.templateName.lower() != sdCps[2] and obj.templateName.lower() != sdCps[3] and obj.cp_getParam('unableToChangeTeam') != 1:
					obj.cp_setParam('flag', sdDefendingTeam)
					obj.cp_setParam('team', sdDefendingTeam)
					obj.flagPosition = Bottom

				radius = float(obj.getTemplateProperty('radius'))
				isHemi = int(obj.cp_getParam('isHemisphere'))

				if isHemi != 0:
					id = bf2.triggerManager.createHemiSphericalTrigger(obj, onCPTrigger, '<<PCO>>', radius, (1, 2, 3))
				else:
					id = bf2.triggerManager.createRadiusTrigger(obj, onCPTrigger, '<<PCO>>', radius, (1, 2, 3))			
				obj.triggerId = id
				obj.lastAttackingTeam = 0
				if obj.cp_getParam('team') > 0:
					obj.flagPosition = Top
				else:
					obj.flagPosition = Bottom			

		host.registerHandler('ControlPointChangedOwner', onCPStatusChange)

		# setup ticket system
		ticketsTeam1 = calcStartTickets(bf2.gameLogic.getDefaultTickets(1))
		ticketsTeam2 = calcStartTickets(bf2.gameLogic.getDefaultTickets(2))

		if gameMode == "sd":
			# setup ticket system
			if sdAttackingTeam == 1:
				ticketsTeam1 = ticketsTeam1 - (ticketsTeam1 * (sdAttackerTicketFactor / 100))
			elif sdAttackingTeam == 2:
				ticketsTeam2 = ticketsTeam2 - (ticketsTeam2 * (sdAttackerTicketFactor / 100))
		
		bf2.gameLogic.setTickets(1, ticketsTeam1)
		bf2.gameLogic.setTickets(2, ticketsTeam2)

		bf2.gameLogic.setTicketState(1, 0)
		bf2.gameLogic.setTicketState(2, 0)

		bf2.gameLogic.setTicketLimit(1, 1, 0)
		bf2.gameLogic.setTicketLimit(2, 1, 0)
		bf2.gameLogic.setTicketLimit(1, 2, 10)
		bf2.gameLogic.setTicketLimit(2, 2, 10)
		bf2.gameLogic.setTicketLimit(1, 3, int(ticketsTeam1*0.1))
		bf2.gameLogic.setTicketLimit(2, 3, int(ticketsTeam2*0.1))
		bf2.gameLogic.setTicketLimit(1, 4, int(ticketsTeam1*0.2))
		bf2.gameLogic.setTicketLimit(2, 4, int(ticketsTeam1*0.2))
		
		host.registerHandler('TicketLimitReached', onTicketLimitReached)
		updateTicketLoss()

		# player events
		host.registerHandler('PlayerDeath', onPlayerDeathCQ)
		host.registerHandler('PlayerKilled', onPlayerKilledCQ)
		host.registerHandler('PlayerRevived', onPlayerRevived)
		host.registerHandler('PlayerSpawn', onPlayerSpawn)
		host.registerHandler('EnterVehicle', onEnterVehicle)
		host.registerHandler('ExitVehicle', onExitVehicle)

		if gameMode == "sd":
			host.registerHandler('VehicleDestroyed', onVehicleDestroyed)
			host.registerHandler('PlayerRepairPoint', onPlayerRepairPoint)

		if g_debug: print "Conquest gamemode initialized."

	elif status == bf2.GameStatus.EndGame: 
		if initialGameMode == "both":
			if gameMode == "aas":
				sdAttackingTeam = aasWinner
				gameMode = "sd"
			else:
				gameMode = "aas"
			
	else:
		bf2.triggerManager.destroyAllTriggers()
		g_controlPoints = []



def calcStartTickets(mapDefaultTickets):
	
	if gameMode == "aas":
		noRatio = aasNoRatio
	else:
		noRatio = sdNoRatio

	if noRatio == True:
		startTickets = bf2.serverSettings.getTicketRatio()
	else:
		startTickets = int(mapDefaultTickets * (bf2.serverSettings.getTicketRatio() / 100.0))

	return startTickets
	
	
	
def onTimeLimitReached(value):
	team1tickets = bf2.gameLogic.getTickets(1)
	team2tickets = bf2.gameLogic.getTickets(2)
	
	winner = 0
	victoryType = 0

	if gameMode == "aas":
		if team1tickets > team2tickets:
			winner = 1
			victoryType = 3
		elif team2tickets > team1tickets:
			winner = 2
			victoryType = 3
	else:
		if sdAttackingTeam > 0:
			if team1tickets > team2tickets:
				winner = 1
				victoryType = 3
			elif team2tickets > team1tickets:
				winner = 2
				victoryType = 3
		else:
			if sdAttackingTeam == 1:
				winner = 2
			else:
				winner = 1	

	EndGame(winner, victoryType)

# The game has been ended.
def EndGame(win, vic):
	global aasWinner
	aasWinner = win

	host.sgl_endGame(win, vic)

# Check to see if objectives (vehicles) have been destroyed.
def onVehicleDestroyed(vehicle, attacker):
	
	if sdDebug == 1:
		host.rcon_invoke('game.sayall destoyed-' + vehicle.templateName)
	
	if vehicle.isPlayerControlObject and vehicle.getIsRemoteControlled():
		if sdAttackingTeam == 2 or sdAttackingTeam == 0:
			if anyObjectivesRemaining(1, vehicle.templateName) == False:
				EndGame(2, 3)

		if sdAttackingTeam == 1 or sdAttackingTeam == 0:
			if anyObjectivesRemaining(2, vehicle.templateName) == False:
				EndGame(1, 3)
	
		

# See if there are any objectives remaining for the given team.
# The objective that was just destroyed won't register yet, so needs to be passed manually.
def anyObjectivesRemaining(team, just_destroyed):

	if team == 1:
		objectives = sdTeam1Objectives
		num_objectives = sdNumTeam1Objectives
	else:
		objectives = sdTeam2Objectives
		num_objectives = sdNumTeam2Objectives

	# An objective has just been destroyed and passed to this function, so start with 1.
	destroyed_objectives = 1
	for k in range(0, 3):
		if objectives[k] != "":
			for obj in bf2.objectManager.getObjectsOfTemplate(objectives[k]):
				if obj.getTeam() != team:
					continue
					
				if obj.getIsWreck() == True:
					destroyed_objectives = destroyed_objectives + 1

	if sdDebug == 1:
		host.rcon_invoke('game.sayall num_objectives-' + str(num_objectives))
		host.rcon_invoke('game.sayall num_destroyed-' + str(destroyed_objectives))
	
	if destroyed_objectives == num_objectives:
		return False
	else:
		return True

# update ticket system
def updateTicketLoss():
	# Search and destro has no ticket loss.
	if gameMode == "sd":
		return

	areaValueTeam1 = 0
	areaValueTeam2 = 0
	totalAreaValue = 0
	numCpsTeam0 = 0
	numCpsTeam1 = 0
	numCpsTeam2 = 0
	
	# calculate control point area value for each team
	for obj in g_controlPoints:
		team = obj.cp_getParam('team')
		if team == 1:
			areaValueTeam1 += obj.cp_getParam('areaValue', team)
			totalAreaValue += areaValueTeam1
			numCpsTeam1 += 1
		elif team == 2:
			areaValueTeam2 += obj.cp_getParam('areaValue', team)
			totalAreaValue += areaValueTeam2
			numCpsTeam2 += 1
		else:
			numCpsTeam0 += 1
			totalAreaValue += 0
		
	# check if a team has no control points
	if numCpsTeam1 == 0 or numCpsTeam2 == 0:
		if numCpsTeam1 == 0:
			losingTeam = 1
			winningTeam = 2
		else:
			losingTeam = 2
			winningTeam = 1
		
		# check if there is anyone alive
		foundLivingPlayer = False
		for p in bf2.playerManager.getPlayers():
			if p.getTeam() == losingTeam and p.isAlive():
				foundLivingPlayer = True
				break
				
		if not foundLivingPlayer:

			# drop tickets
			ticketLossPerSecond = bf2.gameLogic.getDefaultTicketLossAtEndPerMin()
			bf2.gameLogic.setTicketChangePerSecond(losingTeam, -ticketLossPerSecond)
			bf2.gameLogic.setTicketChangePerSecond(winningTeam, 0)
			
			return

	
	# update ticket loss
	if gameMode == "aas":
		team1AreaOverweight = areaValueTeam1 - areaValueTeam2
		percentualOverweight = 1.0
		if totalAreaValue != 0:
			percentualOverweight = abs(team1AreaOverweight / totalAreaValue)
		
		ticketLossPerSecTeam1 = calcTicketLossForTeam(1, areaValueTeam2, -team1AreaOverweight)
		ticketLossPerSecTeam2 = calcTicketLossForTeam(2, areaValueTeam1, team1AreaOverweight)
		bf2.gameLogic.setTicketChangePerSecond(1, -ticketLossPerSecTeam1)
		bf2.gameLogic.setTicketChangePerSecond(2, -ticketLossPerSecTeam2)


	
# actual ticket loss calculation function
def calcTicketLossForTeam(team, otherTeamAreaValue, otherTeamAreaOverweight):
	if otherTeamAreaValue >= 100 and otherTeamAreaOverweight > 0:
		ticketLossPerSecond = (bf2.gameLogic.getDefaultTicketLossPerMin(team) / 60.0) * (otherTeamAreaOverweight / 100.0)
		return ticketLossPerSecond
	else:
		return 0


	
DOWNWARDS = -1
UPWARDS = 1	

# called when tickets reach a predetermined limit (negativ value means that the tickets have become less than the limit)
def onTicketLimitReached(team, limitId):
	if (limitId == -1):
		if (team == 1):
			winner = 2
		
		elif (team == 2):
			winner = 1
		
		bf2.gameLogic.setTicketState(1, 0)
		bf2.gameLogic.setTicketState(2, 0)
	
		EndGame(winner, 3)		

	# update ticket state
	else:
		updateTicketWarning(team, limitId)

	

# called when the ticket state should be updated (for triggering messages and sounds based on tickets left)
def updateTicketWarning(team, limitId):

	oldTicketState = bf2.gameLogic.getTicketState(team)
	newTicketState = 0
	
	if (oldTicketState >= 10):
		newTicketState = 10		

	if (limitId == -2):
		newTicketState = 10
	
	elif (limitId == 2):
		newTicketState = 0		

	elif (limitId == -3):
		newTicketState += 2

	elif (limitId == -4):
		newTicketState += 1

	if (oldTicketState != newTicketState):
		bf2.gameLogic.setTicketState(team, newTicketState)
		
# called when someone enters or exits cp radius
def onCPTrigger(triggerId, cp, vehicle, enter, userData):
	if not cp.isValid(): return

	if sdDebug == 1 or aasDebug == 1:
		host.rcon_invoke('game.sayall ' + ' "CP: ' + cp.templateName + '"')
		host.rcon_invoke('game.sayall ' + ' "game mode: ' + bf2.serverSettings.getGameMode() + '"')

	# can this cp be captured at all?
	if cp.cp_getParam('unableToChangeTeam') != 0:
		return					
		
	if enter:
		for p in vehicle.getOccupyingPlayers():
			cp = getOccupyingCP(p)
			if cp != None:
				if not p.getIsInsideCP():
					if g_debug: print "Resetting enterPctAt for player ", p.getName()
					p.enterCpAt = host.timer_getWallTime()
	
	if vehicle:	
		for p in vehicle.getOccupyingPlayers():
			p.setIsInsideCP(enter)			

	# count people in radius
	team1Occupants = 0
	team2Occupants = 0

	pcos = bf2.triggerManager.getObjects(cp.triggerId)
	for o in pcos:
		if not o: continue # you can get None in the result tuple when the host can't figure out what object left the trigger
		if o.getParent(): continue # getOccupyingPlayers returns all players downwards in the hierarchy, so dont count them twice
		for p in o.getOccupyingPlayers():
			if p.isAlive() and not p.isManDown():
				
				if not p.killed:
					if p.getTeam() == 1:
						team1Occupants += 1
					elif p.getTeam() == 2:
						team2Occupants += 1


	# determine who is taking control
	team1OverWeight = team1Occupants - team2Occupants
	attackOverWeight = 0

	if team1OverWeight > 0:
		attackingTeam = 1
	elif team1OverWeight < 0:
		attackingTeam = 2
	else:
		attackingTeam = 0

	
	if team1Occupants == 0 and team2Occupants == 0:

		# nobody here, slowly go back to owning team
		if cp.cp_getParam('team') == 0:
			attackOverWeight = -0.5
		else:
			attackOverWeight = 0.5
			
		timeToChangeControl = cp.cp_getParam('timeToLoseControl')

	else:
		if gameMode == "aas":
			aasTeam = attackingTeam

			aasSetNextCp(aasTeam, 0)

			if cp.cp_getParam('team') != aasTeam:
				if aasNextCp[aasTeam] != cp.templateName.lower():
					aasMsg = 'game.sayTeam ' + str(aasTeam) + ' "' + aasTip + aasCpName(aasNextCp[aasTeam]) + aasL10nText[2] + aasCpName(cp.templateName) + aasTip + '"'
					if aasMsg != aasLast[aasTeam]:
						aasLast[aasTeam] = aasMsg
						host.rcon_invoke(aasMsg)
					return

		else:
			sdTeam = attackingTeam

			if isBase(cp) == False:
				return	
		
		# raise flag if already ours, or at bottom and neutral. Otherwise lower first.
		if cp.cp_getParam('flag') == attackingTeam or (cp.flagPosition == Bottom and cp.cp_getParam('team') == 0):

			# our flag, raise
			attackOverWeight = abs(team1OverWeight)
			timeToChangeControl = cp.cp_getParam('timeToGetControl')
		else:

			# other team raised flag, lower first
			attackOverWeight = - abs(team1OverWeight)
			timeToChangeControl = cp.cp_getParam('timeToLoseControl')
			
	if cp.cp_getParam('onlyTakeableByTeam') != 0 and cp.cp_getParam('onlyTakeableByTeam') != attackingTeam:
		return


	# flag can only be changed when at bottom
	if cp.flagPosition == Bottom:
		cp.cp_setParam('flag', attackingTeam)


	# calculate flag raising/lowering speed
	if timeToChangeControl > 0:
		takeOverChangePerSecond = 1.0 * attackOverWeight / timeToChangeControl
	else:
		takeOverChangePerSecond = 0.0

	if (cp.flagPosition == Top and takeOverChangePerSecond > 0) or (cp.flagPosition == Bottom and takeOverChangePerSecond < 0):
		takeOverChangePerSecond = 0.0

	if abs(takeOverChangePerSecond) > 0:
		cp.flagPosition = Middle
				
	cp.cp_setParam('takeOverChangePerSecond', takeOverChangePerSecond)

			
	
# called when a control point flag reached top or bottom
def onCPStatusChange(cp, top):

	playerId = -1
	takeoverType = -1
	newTeam = -1
	scoringTeam = -1
	
	if top:	cp.flagPosition = Top
	else:   cp.flagPosition = Bottom
	
	# determine capture / neutralize / defend
	if cp.cp_getParam('team') != 0:

		if top:
			# regained flag, do nothing
			pass
			
		else:
			# neutralize
			newTeam = 0
			if cp.cp_getParam('team') == 1:
				scoringTeam = 2
			else:
				scoringTeam = 1
				
			takeoverType = TAKEOVERTYPE_NEUTRALIZE

	else:

		if top:
			# capture
			newTeam = cp.cp_getParam('flag')
			scoringTeam = newTeam
			takeoverType = TAKEOVERTYPE_CAPTURE

		else:
			# hit bottom, but still neutral
			pass

	
	# scoring
	if takeoverType > 0:
		pcos = bf2.triggerManager.getObjects(cp.triggerId)
	
		# count number of players
		scoringPlayers = []
		firstPlayers = []
		for o in pcos:
			for p in o.getOccupyingPlayers():
				if p.isAlive() and not p.isManDown() and p.getTeam() == scoringTeam:
					if len(firstPlayers) == 0 or p.enterCpAt < firstPlayers[0].enterCpAt:
						firstPlayers = [p]
					elif p.enterCpAt == firstPlayers[0].enterCpAt:
						firstPlayers += [p]
					
					if not p in scoringPlayers:
						scoringPlayers += [p]
	
		# deal score
		for p in scoringPlayers:
			oldScore = p.score.score;
			if takeoverType == TAKEOVERTYPE_CAPTURE:
				if p in firstPlayers:
					p.score.cpCaptures += 1
					addScore(p, SCORE_CAPTURE, RPL)
					bf2.gameLogic.sendGameEvent(p, 12, 0) #12 = Conquest, 0 = Capture
					playerId = p.index
				else:
					p.score.cpAssists += 1
					addScore(p, SCORE_CAPTUREASSIST, RPL)
					bf2.gameLogic.sendGameEvent(p, 12, 2) #12 = Conquest, 2 = Assist


			elif takeoverType == TAKEOVERTYPE_NEUTRALIZE:
				if p in firstPlayers:
					p.score.cpNeutralizes += 1
					addScore(p, SCORE_NEUTRALIZE, RPL)
					bf2.gameLogic.sendGameEvent(p, 12, 3) #12 = Conquest, 3 = Neutralize
				else:
					p.score.cpNeutralizeAssists += 1
					addScore(p, SCORE_NEUTRALIZEASSIST, RPL)
					bf2.gameLogic.sendGameEvent(p, 12, 4) #12 = Conquest, 4 = Neutralize assist
					
				

	# immediate ticket loss for opposite team
	enemyTicketLossInstant = cp.cp_getParam('enemyTicketLossWhenCaptured')
	if enemyTicketLossInstant > 0 and newTeam > 0:
		
		if newTeam == 1:
			punishedTeam = 2
		elif newTeam == 2:
			punishedTeam = 1
		
		tickets = bf2.gameLogic.getTickets(punishedTeam)
		tickets -= enemyTicketLossInstant
		bf2.gameLogic.setTickets(punishedTeam, tickets)
	
	
	# update control point	
	cp.cp_setParam('playerId', playerId) #always set player first
	if newTeam != -1 and cp.cp_getParam('team') != newTeam:
		cp.cp_setParam('team', newTeam)
	onCPTrigger(cp.triggerId, cp, 0, 0, 0)
	updateTicketLoss()
	
	if gameMode == "aas":
		for aasTeam in [ 1, 2 ]:
			aasSetNextCp(aasTeam, 0)
			aasMsg = 'game.sayTeam ' + str(aasTeam) + ' "' + aasTip + aasL10nText[3] + aasCpName(aasNextCp[aasTeam]) + aasTip + '"'
			if aasMsg != aasLast[aasTeam]:
				aasLast[aasTeam] = aasMsg
				host.rcon_invoke(aasMsg)
				
				
def onPlayerDeathCQ(victim, vehicle):		

	# punish team with one ticket
	if victim != None:
		team = victim.getTeam()
		teamTickets = bf2.gameLogic.getTickets(team)
		teamTickets -= 1
		bf2.gameLogic.setTickets(team, teamTickets)

	# check if it was the last player
	foundLivingPlayer = False
	for p in bf2.playerManager.getPlayers():
		if p != victim and p.getTeam() == victim.getTeam() and p.isAlive():
			foundLivingPlayer = True
	
	if not foundLivingPlayer:
		updateTicketLoss()
	


def onPlayerKilledCQ(victim, attacker, weapon, assists, object):
	if not victim: 
		return
	
	victim.killed = True
	
	# update flag takeover status if victim was in a CP radius
	cp = getOccupyingCP(victim)
	if cp != None:
		onCPTrigger(-1, cp, victim.getVehicle(), False, None)

		# give defend score if killing enemy within cp radius
		if attacker != None and attacker.getTeam() != victim.getTeam()\
		   and cp.cp_getParam('unableToChangeTeam') == 0 and cp.cp_getParam('onlyTakeableByTeam') == 0:
		
			if cp != None and cp.cp_getParam('team') == attacker.getTeam():
				attacker.score.cpDefends += 1
				addScore(attacker, SCORE_DEFEND, RPL)
				bf2.gameLogic.sendGameEvent(attacker, 12, 1) #12 = Conquest, 1 = Defend
			
			
			
def onPlayerRevived(victim, attacker):

	# update flag takeover status if victim was in a CP radius
	victim.killed = False
	
	cp = getOccupyingCP(victim)
	if cp != None:
		onCPTrigger(-1, cp, victim.getVehicle(), True, None)
	


def onPlayerSpawn(player, soldier):

	player.killed = False

	if gameMode == "aas":
		aasTeam = player.getTeam()

		aasMsg = 'game.sayTeam ' + str(aasTeam) + ' "' + aasTip + aasL10nText[0] + aasTip + '"'
		if aasMsg != aasLast[aasTeam]:
			aasLast[aasTeam] = aasMsg
			host.rcon_invoke(aasMsg)

		aasSetNextCp(aasTeam, 0)

		aasMsg = 'game.sayTeam ' + str(aasTeam) + ' "' + aasTip + aasL10nText[3] + aasCpName(aasNextCp[aasTeam]) + aasTip + '"'
		if aasMsg != aasLast[aasTeam]:
			aasLast[aasTeam] = aasMsg
			host.rcon_invoke(aasMsg)
	else:
		sdTeam = player.getTeam()

		if sdTeam == 1:
			num_objectives = sdNumTeam1Objectives
		else:
			num_objectives = sdNumTeam2Objectives

		if sdAttackingTeam == 1:
			sdTeamObjectives = sdTeam2Objectives
		else:
			sdTeamObjectives = sdTeam1Objectives
			
		sdObjectiveList = ''

		i = 1
		for sdObjName in sdTeamObjectives:
			sdObjName = sdObjName.lower()
		
			if sdObjName != '':
				sdObjectiveList = sdObjectiveList + sdL10nCp[sdObjName]
				if i < num_objectives:
					if i < num_objectives - 1:
						sdObjectiveList = sdObjectiveList + ', '
					else:
						sdObjectiveList = sdObjectiveList + ' and '
			i = i + 1

		if sdAttackingTeam == 0:
			sdMsg = 'game.sayTeam ' + str(sdTeam) + ' "Destroy the enemy objectives while defending your own."'
		else:
			if sdTeam == sdAttackingTeam:
				sdMsg = 'game.sayTeam ' + str(sdTeam) + ' "* Destroy your enemy\'s ' + sdObjectiveList + ' *"'
			else: 
				sdMsg = 'game.sayTeam ' + str(sdTeam) + ' "* Defend your ' + sdObjectiveList + ' *"'

		if sdMsg != sdLast[sdTeam]:
			sdLast[sdTeam] = sdMsg
			host.rcon_invoke(sdMsg)	
	
def onEnterVehicle(player, vehicle, freeSoldier = False):

	# update flag takeover status if player in a CP radius
	cp = getOccupyingCP(player)
	if cp != None:
		player.setIsInsideCP(True)
	else:		
		player.setIsInsideCP(False)



def onExitVehicle(player, vehicle):

	# update flag takeover status if player in a CP radius
	cp = getOccupyingCP(player)
	if cp != None:
		player.setIsInsideCP(True)
	else:		
		player.setIsInsideCP(False)
		
		
		
# find cp that player is occupying, if any		
def getOccupyingCP(player):
	vehicle = player.getVehicle()
	playerPos = vehicle.getPosition()
	
	# find closest CP
	closestCP = None
	if len(g_controlPoints) == 0: return None
	for obj in g_controlPoints:
		distanceTo = getVectorDistance(playerPos, obj.getPosition())
		if closestCP == None or distanceTo < closestCPdist:
			closestCP = obj
			closestCPdist = distanceTo
	
	# is the player in radius?
	pcos = bf2.triggerManager.getObjects(closestCP.triggerId)
	for o in pcos:
		if o == player.getDefaultVehicle():
			# Player is DEFAULT vehicle - this is needed when called from onEnterVehicle
			return closestCP
		else:
			for p in o.getOccupyingPlayers():
				if p == player:
					return closestCP
	
	return None



# get distance between two positions
def getVectorDistance(pos1, pos2):
	diffVec = [0.0, 0.0, 0.0]
	diffVec[0] = math.fabs(pos1[0] - pos2[0])
	diffVec[1] = math.fabs(pos1[1] - pos2[1])
	diffVec[2] = math.fabs(pos1[2] - pos2[2])
	 
	return math.sqrt(diffVec[0] * diffVec[0] + diffVec[1] * diffVec[1] + diffVec[2] * diffVec[2])
		
		
	
